@@ -349,32 +349,29 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             Some(shared_dt) => {
                 match expected_state_dt {
                     Some(state_dt) => quote! {
-                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, ()> {
-                            let shared_d_enc_some = shared_d_enc.ok_or(())?;
+                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                            let shared_d_enc_some = shared_d_enc.ok_or(RestoreError::EmptyData)?;
                             let shared_d: #shared_dt = match shared_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(())?,
-                                _ => return Err(())
+                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
-                            let state_d_enc_some = state_d_enc.ok_or(())?;
+                            let state_d_enc_some = state_d_enc.ok_or(RestoreError::EmptyData)?;
                             let state_d: #state_dt = match state_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(())?,
-                                _ => return Err(())
+                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(state_d), shared_d, observer)))
                         }
                     },
                     None => quote! {
-                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, ()> {
-                            let shared_d_enc_some = shared_d_enc.ok_or(())?;
+                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                            let shared_d_enc_some = shared_d_enc.ok_or(RestoreError::EmptyData)?;
                             let shared_d: #shared_dt = match shared_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(())?,
-                                _ => return Err(())
+                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             if state_d_enc.is_some() {
-                                return Err(())
+                                return Err(RestoreError::UnexpectedData)
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(), shared_d, observer)))
@@ -385,28 +382,27 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             None => {
                 match expected_state_dt {
                     Some(state_dt) => quote! {
-                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, ()> {
+                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             if shared_d_enc.is_some() {
-                                return Err(())
+                                return Err(RestoreError::UnexpectedData)
                             };
 
-                            let state_d_enc_some = state_d_enc.ok_or(())?;
+                            let state_d_enc_some = state_d_enc.ok_or(RestoreError::EmptyData)?;
                             let state_d: #state_dt = match state_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(())?,
-                                _ => return Err(())
+                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(state_d), observer)))
                         }
                     },
                     None => quote! {
-                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, ()> {
+                        fn #fn_name<T: Observer>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             if shared_d_enc.is_some() {
-                                return Err(())
+                                return Err(RestoreError::UnexpectedData)
                             };
 
                             if state_d_enc.is_some() {
-                                return Err(())
+                                return Err(RestoreError::UnexpectedData)
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(), observer)))
@@ -461,6 +457,14 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
         pub enum TransitionError<T> {
             ObserverError(T)
         }
+
+        #[derive(Debug)]
+        pub enum RestoreError {
+            EmptyData,
+            UnexpectedData,
+            InvalidData,
+            InvalidState
+        }
         
         pub enum State {
             #(#state_names),*
@@ -503,10 +507,10 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
 
         #(#restore_fns)*
 
-        pub fn restore<T: Observer>(id: String, state_str: &str, data: Option<Encoded>, state_data: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, ()> {
+        pub fn restore<T: Observer>(id: String, state_str: &str, data: Option<Encoded>, state_data: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
             match state_str {
                 #(#restore_arms,)*
-                _ => Err(())
+                _ => Err(RestoreError::InvalidState)
             }
         }
     };
